@@ -119,13 +119,15 @@ Options:
          Specify the name of the changelog to use in place of debian/changelog
          No directory traversal or checking is performed in this case.
   --news <newsfile>
-         Specify that the newsfile (default debian/NEWS) is to be edited;
-         cannot be used with --changelog
+         Specify that the newsfile (default debian/NEWS) is to be edited
   --[no]multimaint
          When appending an entry to a changelog section (-a), [do not]
          indicate if multiple maintainers are now involved (default: do so)
   -m, --maintmaint
          Don\'t change (maintain) the maintainer details in the changelog entry
+  -t, --mainttrailer
+         Don\'t change (maintain) the trailer line in the changelog entry; i.e.
+         maintain the maintainer and date/time details
   --check-dirname-level N
          How much to check directory names:
          N=0   never
@@ -175,6 +177,7 @@ my $opt_release_heuristic = 'log';
 my $opt_multimaint = 1;
 my $opt_multimaint_merge = 0;
 my $opt_tz = undef;
+my $opt_mainttrailer = 0;
 
 # Next, read configuration files and then command line
 # The next stuff is boilerplate
@@ -244,7 +247,7 @@ if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
 # We use bundling so that the short option behaviour is the same as
 # with older debchange versions.
 my ($opt_help, $opt_version);
-my ($opt_i, $opt_a, $opt_e, $opt_r, $opt_v, $opt_b, $opt_d, $opt_D, $opt_u);
+my ($opt_i, $opt_a, $opt_e, $opt_r, $opt_v, $opt_b, $opt_d, $opt_D, $opt_u, $opt_t);
 my ($opt_n, $opt_security, $opt_qa, $opt_U, $opt_bpo, $opt_c, $opt_m, $opt_create, $opt_package, @closes);
 my ($opt_news);
 my ($opt_ignore, $opt_level, $opt_regex, $opt_noconf);
@@ -279,6 +282,7 @@ GetOptions("help|h" => \$opt_help,
 	   "multimaint!" => \$opt_multimaint,
 	   "multi-maint!" => \$opt_multimaint,
 	   "m|maintmaint" => \$opt_m,
+	   "t|mainttrailer!" => \$opt_t,
 	   "ignore-dirname" => \$opt_ignore,
 	   "check-dirname-level=s" => \$opt_level,
 	   "check-dirname-regex=s" => \$opt_regex,
@@ -300,6 +304,10 @@ if (defined $opt_distributor) {
 }
 $distributor ||= 'Debian';
 
+if (! defined $opt_t && $opt_release_heuristic eq 'changelog') {
+    $opt_t = 1;
+}
+
 if ($opt_noconf) {
     fatal "--no-conf is only acceptable as the first command-line option!";
 }
@@ -320,9 +328,6 @@ if (defined $opt_level) {
 
 if (defined $opt_regex) { $check_dirname_regex = $opt_regex; }
 
-fatal "Only one of -c/--changelog and --news is allowed; try $progname --help for more help"
-    if $opt_c && $opt_news;
-
 # Only allow at most one non-help option
 fatal "Only one of -a, -i, -e, -r, -v, -d, -n/--nmu, -s/--security, --qa, --bpo is allowed;\ntry $progname --help for more help"
     if ($opt_i?1:0) + ($opt_a?1:0) + ($opt_e?1:0) + ($opt_r?1:0) + ($opt_v?1:0) + ($opt_d?1:0) + ($opt_n?1:0) + ($opt_qa?1:0) + ($opt_security?1:0) + ($opt_bpo?1:0) > 1;
@@ -340,8 +345,8 @@ if (defined $opt_D) {
 	    $warnings++;
 	}
     } elsif ($distributor eq 'Ubuntu') {
-	unless ($opt_D =~ /^((warty|hoary|breezy|dapper|edgy|feisty|gutsy)(-updates|-security|-proposed)?|UNRELEASED)$/) {
-	    warn "$progname warning: Recognised distributions are:\n{warty,hoary,breezy,dapper,edgy,feisty,gutsy}{,-updates,-security,-proposed} and UNRELEASED.\nUsing your request anyway.\n";
+	unless ($opt_D =~ /^((warty|hoary|breezy|dapper|edgy|feisty|gutsy|hardy|intrepid)(-updates|-security|-proposed)?|UNRELEASED)$/) {
+	    warn "$progname warning: Recognised distributions are:\n{warty,hoary,breezy,dapper,edgy,feisty,gutsy,hardy,intrepid}{,-updates,-security,-proposed} and UNRELEASED.\nUsing your request anyway.\n";
 	    $warnings++;
 	}
     } else {
@@ -356,6 +361,7 @@ fatal "--package can only be used with --create"
     if $opt_package && ! $opt_create;
 
 my $changelog_path = $opt_c || $ENV{'CHANGELOG'} || 'debian/changelog';
+my $real_changelog_path = $changelog_path;
 if ($opt_news) { $changelog_path = $opt_news; }
 if ($changelog_path ne 'debian/changelog' and not $opt_news) {
     $check_dirname_level = 0;
@@ -477,8 +483,8 @@ if (! $opt_create || ($opt_create && $opt_news)) {
     fatal "No distribution in changelog!"
 	unless exists $changelog{'Distribution'};
     if ($distributor eq 'Ubuntu') {
-	# In Ubuntu uploads should go to gutsy
-	$DISTRIBUTION = 'gutsy';
+	# In Ubuntu uploads should go to hardy
+	$DISTRIBUTION = 'hardy';
     } else {
 	$DISTRIBUTION=$changelog{'Distribution'};
     }
@@ -486,6 +492,22 @@ if (! $opt_create || ($opt_create && $opt_news)) {
 	unless exists $changelog{'Changes'};
     $CHANGES=$changelog{'Changes'};
 
+    # Find the current package version
+    if ($opt_news) {
+	open PARSED, qq[dpkg-parsechangelog -l"$real_changelog_path" | ]
+	    or fatal "Cannot execute dpkg-parsechangelog: $!";
+	while (<PARSED>) {
+	    chomp;
+	    if (m%^Version:\s+(\S+)$%) {
+		$VERSION = $1;
+		last;
+	    }
+	}
+	close PARSED
+	    or fatal "Problem executing dpkg-parsechangelog: $!";
+	if ($?) { fatal "dpkg-parsechangelog failed!"; }
+    }
+
     # Is the directory name acceptable?
     if ($check_dirname_level ==  2 or
 	($check_dirname_level == 1 and $chdir)) {
@@ -654,7 +676,7 @@ if (@closes and $opt_query) { # and we have to query the BTS
 		$lastbug=$1;
 	    }
 	    elsif (defined $lastbug and
-		   m%<a(?: class=\".*?\")? href=\"pkgreport.cgi\?pkg=([a-z0-9\+\-\.]*)%i) {
+		   m%<a(?: class=\".*?\")? href=\"pkgreport.cgi\?(?:[^\"]*?;)(?:(pkg|package))=([a-z0-9\+\-\.]*)%i) {
 		push @{$bugs{$lastbug}}, $1
 		    if exists $bugs{$lastbug};
 		$lastbug = undef;
@@ -680,7 +702,7 @@ if (@closes and $opt_query) { # and we have to query the BTS
 		} else {
 		    my ($bugtitle) = ($bug =~ m%<TITLE>.*?\#$close - (.*?)</TITLE>%);
 		    $bugtitle ||= '';
-		    my ($bugpkg) = ($bug =~ m%<a(?: class=\".*?\")? href=\"pkgreport.cgi\?pkg=([a-z0-9\+\-\.]*)%i);
+		    my ($bugpkg) = ($bug =~ m%<a(?: class=\".*?\")? href=\"pkgreport.cgi\?(?:[^\"]*?;)(?:(pkg|package))=([a-z0-9\+\-\.]*)%i);
 		    $bugpkg ||= '?';
 		    my ($bugsrcpkg) = ($bug =~ m%<a(?: class=\".*?\")? href=\"pkgreport.cgi\?src=([a-z0-9\+\-\.]*)%i);
 		    $bugsrcpkg ||= '?';
@@ -720,12 +742,18 @@ if (@closes and ! $opt_query) { # and we don't have to query the BTS
     }
 }
 
-
 # Get a possible changelog entry from the command line
-my $TEXT=decode_utf8(join(' ', @ARGV));
+my $ARGS=join(' ', @ARGV);
+my $TEXT=decode_utf8($ARGS);
+my $EMPTY_TEXT=0;
+
 if (@ARGV and ! $TEXT) {
-    warn "$progname warning: command-line changelog entry not UTF-8 encoded; ignoring\n";
-    $TEXT='';
+    if ($ARGS) {
+	warn "$progname warning: command-line changelog entry not UTF-8 encoded; ignoring\n";
+	$TEXT='';
+    } else {
+	$EMPTY_TEXT = 1;
+    }
 }
 
 # Get the date
@@ -790,7 +818,9 @@ my $tmpchk=1;
 my ($NEW_VERSION, $NEW_SVERSION, $NEW_UVERSION);
 my $line;
 
-if (($opt_i || $opt_n || $opt_security || $opt_qa || $opt_bpo || $opt_v || $opt_d) && ! $opt_create) {
+if (($opt_i || $opt_n || $opt_security || $opt_qa || $opt_bpo || $opt_v || $opt_d ||
+    ($opt_news && $VERSION ne $changelog{'Version'})) && ! $opt_create) {
+
     # Check that a given explicit version number is sensible.
     if ($opt_v || $opt_d) {
 	if($opt_v) {
@@ -875,11 +905,15 @@ if (($opt_i || $opt_n || $opt_security || $opt_qa || $opt_bpo || $opt_v || $opt_
 	    } elsif ($opt_bpo and not $start =~ /~bpo\.$/) {
 		# If it's not already a backport make it so
 		# otherwise we can be safe if we behave like dch -i
-		$end .= "~bpo.1";
-	    } else {
+		$end .= "~bpo40+1";
+	    } elsif (!$opt_news) {
+		# Don't bump the version of a NEWS file in this case as we're
+		# using the version from the changelog
 		$end++;
 	    }
 	    $DISTRIBUTION=$changelog{'Distribution'} if $start =~ /prop(osed)?(\d+\.)*$/;
+	    } elsif ($start =~ /~ppa([0-9]*)$/) {
+		$end++
 	    } else {
 		$end .= "ubuntu1";
 	    }
@@ -940,7 +974,8 @@ elsif (($opt_r || $opt_a) && ! $opt_create) {
     # last entry, and determine whether there are existing
     # multi-developer changes by the current maintainer.
     $line=-1;
-    my ($lastmaint, $nextmaint, $maintline, $count);
+    my ($lastmaint, $nextmaint, $maintline, $count, $lastheader, $lastdist);
+    my $savedline = $line;;
     while (<S>) {
 	$line++;
 	# Start of existing changes by the current maintainer
@@ -954,10 +989,21 @@ elsif (($opt_r || $opt_a) && ! $opt_create) {
 	    # maintainer
 	    $nextmaint ||= $1;
 	}
-
-	if (/^ --\s+([^<]+)\s+/) {
+	elsif (defined $lastmaint) {
+	    if (m/^\w[-+0-9a-z.]* \([^\(\) \t]+\)((?:\s+[-+0-9a-z.]+)+)\;\s+urgency=(\w+)/i) {
+		$lastheader = $_;
+		$lastdist = $1;
+		$lastdist =~ s/^\s+//;
+		undef $lastdist if $lastdist eq "UNRELEASED";
+		# Revert to our previously saved position
+		$line = $savedline;
+		last;
+	    }
+	}	
+	elsif (/^ --\s+([^<]+)\s+/) {
 	    $lastmaint=$1;
-	    last;
+	    # Remember where we are so we can skip back afterwards
+	    $savedline = $line;
 	}
 
 	if (defined $maintline && !defined $nextmaint) {
@@ -1002,14 +1048,15 @@ elsif (($opt_r || $opt_a) && ! $opt_create) {
 	if ($opt_r) {
 	    # Change the distribution from UNRELEASED for release.
 	    if ($distributor eq 'Ubuntu') {
-		$distribution = $opt_D || "gutsy";
+		$distribution = $opt_D || "hardy";
 	    } else {
-		$distribution = $opt_D || "unstable";
+		$distribution = $opt_D || $lastdist || "unstable";
 	    }
 	    # Set the start-line to 1, as we don't know what they want to edit
 	    $line=1;
+	} else {
+	    $distribution = $opt_D if $opt_D;
 	}
-	$distribution = $opt_D if $opt_D;
 	$urgency = $opt_u if $opt_u;
 	$CHANGES =~ s/^(\w[-+0-9a-z.]* \([^\(\) \t]+\))(?:\s+[-+0-9a-z.]+)+\;\s+urgency=\w+/$1 $distribution; urgency=$urgency/i;
     } else {
@@ -1047,7 +1094,7 @@ elsif (($opt_r || $opt_a) && ! $opt_create) {
 	} elsif ($opt_news) {
 	    print O "\n  \n";
 	    $line++;
-	} else {
+	} elsif (!$EMPTY_TEXT) {
 	    print O "  * \n";
 	}
     }
@@ -1062,7 +1109,15 @@ elsif (($opt_r || $opt_a) && ! $opt_create) {
 	}
     }
 
-    print O "\n -- $MAINTAINER <$EMAIL>  $DATE\n";
+    if ($opt_t && $opt_a) {
+	print O "\n -- $changelog{'Maintainer'}  $changelog{'Date'}\n";
+    } else {
+	print O "\n -- $MAINTAINER <$EMAIL>  $DATE\n";
+    }
+
+    if ($lastheader) {
+	print O "\n$lastheader";
+    }
 
     # Copy the rest of the changelog file to new one
     # Slurp the rest....
@@ -1075,7 +1130,11 @@ elsif ($opt_e && ! $opt_create) {
 
     print O $CHANGES;
 
-    print O "\n -- $MAINTAINER <$EMAIL>  $DATE\n";
+    if ($opt_t) {
+	print O "\n -- $changelog{'Maintainer'}  $changelog{'Date'}\n";
+    } else {
+	print O "\n -- $MAINTAINER <$EMAIL>  $DATE\n";
+    }
 
     # Copy the rest of the changelog file to the new one
     $line=-1;
@@ -1127,7 +1186,7 @@ if ($warnings) {
 }
 
 # Now Run the Editor; always run if doing "closes" to give a chance to check
-if (! $TEXT or @closes_text or $opt_create) {
+if ((!$TEXT and !$EMPTY_TEXT) or @closes_text or $opt_create) {
     my $mtime = (stat("$changelog_path.dch"))[9];
     defined $mtime or fatal
 	"Error getting modification time of temporary $changelog_path: $!";
